@@ -17,7 +17,7 @@ from abc import abstractmethod
 
 import numpy as np
 
-from joblib import Parallel, delayed
+from joblib import Parallel
 
 from ..base import ClassifierMixin
 from ..base import RegressorMixin
@@ -33,6 +33,7 @@ from ..utils.validation import column_or_1d
 from ..utils.validation import _deprecate_positional_args
 from ..exceptions import NotFittedError
 from ..utils._estimator_html_repr import _VisualBlock
+from ..utils.fixes import delayed
 
 
 class _BaseVoting(TransformerMixin, _BaseHeterogeneousEnsemble):
@@ -53,7 +54,7 @@ class _BaseVoting(TransformerMixin, _BaseHeterogeneousEnsemble):
         if self.weights is None:
             return None
         return [w for est, w in zip(self.estimators, self.weights)
-                if est[1] not in (None, 'drop')]
+                if est[1] != 'drop']
 
     def _predict(self, X):
         """Collect results from clf.predict calls."""
@@ -78,19 +79,43 @@ class _BaseVoting(TransformerMixin, _BaseHeterogeneousEnsemble):
                         message=self._log_message(names[idx],
                                                   idx + 1, len(clfs))
                 )
-                for idx, clf in enumerate(clfs) if clf not in (None, 'drop')
+                for idx, clf in enumerate(clfs) if clf != 'drop'
             )
 
         self.named_estimators_ = Bunch()
 
-        # Uses None or 'drop' as placeholder for dropped estimators
+        # Uses 'drop' as placeholder for dropped estimators
         est_iter = iter(self.estimators_)
         for name, est in self.estimators:
-            current_est = est if est in (None, 'drop') else next(est_iter)
+            current_est = est if est == 'drop' else next(est_iter)
             self.named_estimators_[name] = current_est
 
         return self
 
+    def fit_transform(self, X, y=None, **fit_params):
+        """Return class labels or probabilities for each estimator.
+
+        Return predictions for X for each estimator.
+
+        Parameters
+        ----------
+        X : {array-like, sparse matrix, dataframe} of shape \
+                (n_samples, n_features)
+            Input samples
+
+        y : ndarray of shape (n_samples,), default=None
+            Target values (None for unsupervised transformations).
+
+        **fit_params : dict
+            Additional fit parameters.
+
+        Returns
+        -------
+        X_new : ndarray array of shape (n_samples, n_features_new)
+            Transformed array.
+        """
+        return super().fit_transform(X, y, **fit_params)
+
     @property
     def n_features_in_(self):
         # For consistency with other estimators we raise a AttributeError so
@@ -109,14 +134,17 @@ class _BaseVoting(TransformerMixin, _BaseHeterogeneousEnsemble):
         names, estimators = zip(*self.estimators)
         return _VisualBlock('parallel', estimators, names=names)
 
+    def _more_tags(self):
+        return {"preserves_dtype": []}
+
 
 class VotingClassifier(ClassifierMixin, _BaseVoting):
     """Soft Voting/Majority Rule classifier for unfitted estimators.
 
-    .. versionadded:: 0.17
-
     Read more in the :ref:`User Guide <voting_classifier>`.
 
+    .. versionadded:: 0.17
+
     Parameters
     ----------
     estimators : list of (str, estimator) tuples
@@ -126,11 +154,8 @@ class VotingClassifier(ClassifierMixin, _BaseVoting):
         using ``set_params``.
 
         .. versionchanged:: 0.21
-            ``'drop'`` is accepted.
-
-        .. deprecated:: 0.22
-           Using ``None`` to drop an estimator is deprecated in 0.22 and
-           support will be dropped in 0.24. Use the string ``'drop'`` instead.
+            ``'drop'`` is accepted. Using None was deprecated in 0.22 and
+            support was removed in 0.24.
 
     voting : {'hard', 'soft'}, default='hard'
         If 'hard', uses predicted class labels for majority rule voting.
@@ -162,6 +187,8 @@ class VotingClassifier(ClassifierMixin, _BaseVoting):
         If True, the time elapsed while fitting will be printed as it
         is completed.
 
+        .. versionadded:: 0.23
+
     Attributes
     ----------
     estimators_ : list of classifiers
@@ -178,7 +205,7 @@ class VotingClassifier(ClassifierMixin, _BaseVoting):
 
     See Also
     --------
-    VotingRegressor: Prediction voting regressor.
+    VotingRegressor : Prediction voting regressor.
 
     Examples
     --------
@@ -359,14 +386,14 @@ class VotingClassifier(ClassifierMixin, _BaseVoting):
 class VotingRegressor(RegressorMixin, _BaseVoting):
     """Prediction voting regressor for unfitted estimators.
 
-    .. versionadded:: 0.21
-
     A voting regressor is an ensemble meta-estimator that fits several base
     regressors, each on the whole dataset. Then it averages the individual
     predictions to form a final prediction.
 
     Read more in the :ref:`User Guide <voting_regressor>`.
 
+    .. versionadded:: 0.21
+
     Parameters
     ----------
     estimators : list of (str, estimator) tuples
@@ -376,11 +403,8 @@ class VotingRegressor(RegressorMixin, _BaseVoting):
         ``set_params``.
 
         .. versionchanged:: 0.21
-            ``'drop'`` is accepted.
-
-        .. deprecated:: 0.22
-           Using ``None`` to drop an estimator is deprecated in 0.22 and
-           support will be dropped in 0.24. Use the string ``'drop'`` instead.
+            ``'drop'`` is accepted. Using None was deprecated in 0.22 and
+            support was removed in 0.24.
 
     weights : array-like of shape (n_regressors,), default=None
         Sequence of weights (`float` or `int`) to weight the occurrences of
@@ -396,6 +420,8 @@ class VotingRegressor(RegressorMixin, _BaseVoting):
         If True, the time elapsed while fitting will be printed as it
         is completed.
 
+        .. versionadded:: 0.23
+
     Attributes
     ----------
     estimators_ : list of regressors
@@ -409,7 +435,7 @@ class VotingRegressor(RegressorMixin, _BaseVoting):
 
     See Also
     --------
-    VotingClassifier: Soft Voting/Majority Rule classifier.
+    VotingClassifier : Soft Voting/Majority Rule classifier.
 
     Examples
     --------
